/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or sbbuild/licenses/cddl-gpl-2-cp.txt. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * sbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Portions Copyrighted 2008-2015 George Waldon
 */
package org.starbeans.modules.corelite.actions;

import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.BeanInfo;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.*;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import org.openide.ErrorManager;
import org.openide.filesystems.FileUtil;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import org.openide.util.actions.Presenter.Popup;
import org.openide.util.ContextAwareAction;
import org.openide.util.actions.Presenter;
import org.openide.awt.Mnemonics;
import org.openide.util.*;
import org.openide.util.NbBundle.Messages;
import org.starbeans.modules.corelite.utilities.UIUtilities;
import org.starbeans.modules.corelite.ui.NewFileWizard;
import static org.starbeans.modules.corelite.actions.Bundle.*;

/** Action for invoking the project sensitive NewFile Wizard
 */
//from org.netbeans.modules.project.ui.actions.NewFile in projectui
@Messages({
    "LBL_NewFileAction_Name=&New File...",
    "LBL_NewFileAction_PopupName=New",
    "#{0} Name of the template", "LBL_NewFileAction_Template_PopupName={0}..."
})
public class NewFile extends LookupSensitiveAction implements PropertyChangeListener,
        Popup, PopupMenuListener, Presenter.Menu, ContextAwareAction {

    private static final RequestProcessor RP = new RequestProcessor(NewFile.class);
    private static final RequestProcessor INSTANTIATE_RP = new RequestProcessor(NewFile.class.getName() + ".INSTANTIATE_RP", 5);
    private static final Logger LOG = Logger.getLogger(NewFile.class.getName());
    private static DataObject[] templates;
    private JMenuItem menuPresenter;
    private JMenu subMenu;

    public NewFile() {
        this( null );
    }

    @Messages("LBL_NewFileAction_Tooltip=New File...")
    public NewFile( Lookup context ) {
        super(ImageUtilities.loadImageIcon("org/starbeans/modules/corelite/resources/newFile.gif", false), context, new Class[]{DataObject.class});
        setDisplayName(LBL_NewFileAction_Name());
        putValue(SHORT_DESCRIPTION, LBL_NewFileAction_Tooltip());
        /*STARBEANS*/ //Since 7.2 to have large icon in toolbar!
        putValue("iconBase", "org/starbeans/modules/corelite/resources/newFile.gif"); //NOI18N
        refresh(getLookup(), true);
    }

    @Override
    protected void refresh(final Lookup context, boolean immediate) {
        // #59615: update synch if possible; only replan if not already in EQ.
        Mutex.EVENT.readAccess(new Runnable() {

            @Override
            public void run() {
                DataFolder preselectedFolder = preselectedFolder(context);
                if (preselectedFolder == null) {
                    setEnabled(false);
                    return;
                }
                FileObject fo = preselectedFolder.getPrimaryFile();
                if (fo == null) {
                    setEnabled(false);
                    return;
                }

                try {
                    org.openide.filesystems.FileSystem fs = fo.getFileSystem();
                    if (fs.isDefault()) {
                        setEnabled(false);
                        return;
                    }
                } catch (Exception e) {
                    setEnabled(false);
                    return;
                }
                File fileForFolder = FileUtil.toFile(fo);
                if (fileForFolder == null || !fileForFolder.exists()) {
                    setEnabled(false);
                    return;
                }
                setEnabled(preselectedFolder != null);
                setDisplayName(LBL_NewFileAction_Name());
            }
        });
    } 
    
    @Override
    public JMenuItem getPopupPresenter() {
        return getMenuPresenter();
    }
        

    @Override
    protected void actionPerformed( Lookup context ) {
        doPerform(context, null);
    }

    private void doPerform(Lookup context, final DataObject template) {

        if ( context == null ) {
            context = getLookup();
        }

        final NewFileWizard wd = new NewFileWizard();

        DataFolder preselectedFolder = preselectedFolder(context);
        if (preselectedFolder != null) {
            wd.setTargetFolder(preselectedFolder);
        }
        
        INSTANTIATE_RP.post(new Runnable() {
            @Override public void run() {
        try {
            Set resultSet = template == null ? wd.instantiate () : wd.instantiate( template );

            if (resultSet == null || resultSet.isEmpty ()) {
                // no new object, no work
                return ;
            }

            Iterator it = resultSet.iterator ();

            while (it.hasNext ()) {
                Object obj = it.next ();
                DataObject newDO = null;
                if (obj instanceof DataObject) {
                    newDO = (DataObject) obj;
                } else if (obj instanceof FileObject) {
                    try {
                        newDO = DataObject.find ((FileObject) obj);
                    } catch (DataObjectNotFoundException x) {
                        // XXX
                        assert false : obj;
                    }
                } else {
                    assert false : obj;
                }
                if (newDO != null) {
                    UIUtilities.openAndSelectNewObject (newDO);
                }
            }
        } catch (IOException e) {
            ErrorManager.getDefault().notify(ErrorManager.INFORMATIONAL, e);
        }
            }
        });
    }

    // Context Aware action implementation -------------------------------------

    @Override
    public Action createContextAwareInstance( Lookup actionContext ) {
        return new NewFile( actionContext );
    }

    private void fillSubMenu() {
        fillSubMenu(subMenu);
    }

    // Private methods ---------------------------------------------------------
    private DataFolder preselectedFolder(Lookup context) {

        DataFolder preselectedFolder = null;

        // Try to find selected folder
        preselectedFolder = context.lookup(DataFolder.class);
        if ( preselectedFolder == null ) {
            // No folder selectd try with DataObject
            DataObject dobj = context.lookup(DataObject.class);
            if ( dobj != null) {
                // DataObject found => we'll use the parent folder
                preselectedFolder = dobj.getFolder();
            } else {
                // fallback
                FileObject fo = context.lookup(FileObject.class);
                if(fo != null) {
                    if(fo.isFolder()) {
                        preselectedFolder = DataFolder.findFolder(fo);
                    } else if(fo.getParent() != null) {                        
                        preselectedFolder = DataFolder.findFolder(fo.getParent());                    
                    }
                }
            }
        }

        LOG.log(Level.FINE, "#210919: found preselected folder {0} for {1}", new Object[] {preselectedFolder, context});
        return preselectedFolder;
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        refresh(Lookup.EMPTY, false);
    }
    public static String TEMPLATE_PROPERTY = "org.starbeans.modules.corelite.actions.NewFile.Template"; //NOI18N

    @Messages({
        "LBL_NewFileAction_File_PopupName=File/Folder...",
        "NewFile.please_wait=Please wait..."
    })
    private void fillSubMenu(final JMenu menuItem) {
        menuItem.removeAll();
        JMenuItem wait = new JMenuItem(NewFile_please_wait());
        wait.setEnabled(false);
        menuItem.add(wait);
        RP.post(new Runnable() {

            @Override
            public void run() {
                EventQueue.invokeLater(new Runnable() {

                    @Override
                    public void run() {
                        menuItem.removeAll();

                        ActionListener menuListener = new PopupListener();
                        DataFolder preselectedFolder = preselectedFolder(getLookup());
                        boolean canWrite;
                        if (preselectedFolder == null) {
                            canWrite = false;
                        } else {
                            FileObject pf = preselectedFolder.getPrimaryFile();
                            canWrite = pf != null && pf.canWrite();
                        }

                        DataObject ltemplates[] = getTemplates();
                        for (int i = 0; i < ltemplates.length; i++) {
                            Node n = ltemplates[i].getNodeDelegate();
                            JMenuItem item = new JMenuItem(
                                    LBL_NewFileAction_Template_PopupName(n.getDisplayName()),
                                    new ImageIcon(n.getIcon(BeanInfo.ICON_COLOR_16x16)));
                            item.addActionListener(menuListener);
                            item.putClientProperty(TEMPLATE_PROPERTY, ltemplates[i]);

                            item.setEnabled(canWrite);
                            menuItem.add(item);
                            // #205616 - need to refresh please wait node
                            menuItem.getPopupMenu().pack();
                        }        
                    }
                });
            }
        });
    }
    
    

    private DataObject[] getTemplates() {

        if (templates == null) {

            ArrayList<DataObject> tList = new ArrayList<DataObject>(2);
            DataObject template;

            template = findTemplate("Templates/Other/file"); //NOI18N
            if (template != null) {
                tList.add(template);
            }

            template = findTemplate("Templates/Other/Folder"); //NOI18N
            if (template != null) {
                tList.add(template);
            }

            templates = new DataObject[tList.size()];
            tList.toArray(templates);
        }
        return templates;
    }

    private static DataObject findTemplate(String name) {
        FileObject tFo = FileUtil.getConfigFile(name);
        if (tFo == null) {
            return null;
        }
        try {
            return DataObject.find(tFo);
        } catch (DataObjectNotFoundException e) {
            return null;
        }

    }

    private class PopupListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            JMenuItem source = (JMenuItem) e.getSource();
            DataObject template = (DataObject) source.getClientProperty(TEMPLATE_PROPERTY);
            doPerform(null, template);
        }
    }

    // Implementation of PopupMenuListener -------------------------------------
    @Override
    public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
        fillSubMenu();
    }

    @Override
    public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
    }

    @Override
    public void popupMenuCanceled(PopupMenuEvent e) {
    }

    // Implementation of Presenter.Menu ----------------------------------------
    @Override
    public JMenuItem getMenuPresenter() {

        if (menuPresenter == null) {
            menuPresenter = new JMenuItem(this);

            Icon icon = null;
            // ignore icon if noIconInMenu flag is set
            if (!Boolean.TRUE.equals(getValue("noIconInMenu"))) { //NOI18N
                icon = (Icon) getValue(Action.SMALL_ICON);
            }
            if (icon == null) {
                icon = new ImageIcon(ImageUtilities.loadImage("org/starbeans/modules/corelite/resources/empty.gif")); //NOI18N
            }
            Mnemonics.setLocalizedText(menuPresenter, LBL_NewFileAction_Name());
            menuPresenter.setIcon(icon);
        }
        return menuPresenter;
    }
}
    
    
    